@@ -2,13 +2,14 @@
 #
 # Author: Yipeng Sun <syp at umd dot edu>
 # License: BSD 2-clause
-# Last Change: Sun Mar 14, 2021 at 02:49 AM +0100
+# Last Change: Sat Jun 26, 2021 at 12:00 AM +0200
 
 import re
 import logging
 
 from collections import defaultdict, Counter
 from dataclasses import dataclass
+from copy import deepcopy
 
 from pyBabyMaker.base import TermColor as TC
 from pyBabyMaker.base import UniqueList, BaseMaker
@@ -57,6 +58,15 @@ class BabyVariableResolver(VariableResolver):
         var.fname = scope+'_'+var.name
         return var
 
+    @staticmethod
+    def unpack_resolved(var):
+        """
+        Unpack resolved variable to (scope, original variable) tuple for
+        ``babymaker``.
+        """
+        scope = var.fname.split('_')[0]
+        return scope, var.name
+
 
 ########################
 # Configuration parser #
@@ -66,13 +76,15 @@ class BabyConfigParser:
     """
     Basic parser for YAML C++ code instruction.
     """
-    def __init__(self, parsed_config, dumped_ntuple, debug=False):
+    def __init__(self, parsed_config, dumped_ntuple,
+                 literals={}, debug=False):
         """
         Initialize the config parser with parsed YAML file and dumped ntuple
         structure.
         """
         self.parsed_config = parsed_config
         self.dumped_ntuple = dumped_ntuple
+        self.literals = literals
         self.debug = debug
 
         if debug:
@@ -88,8 +100,11 @@ class BabyConfigParser:
             'system_headers': UniqueList(),
             'user_headers': UniqueList(),
             'trees': {},
+            'input_trees': UniqueList(),
         }
         self.parse_headers(self.parsed_config, directive)
+        parsed_literals = {k: BabyVariable(k, literal=v)
+                           for k, v in self.literals.items()}
 
         for output_tree, config in self.parsed_config['output'].items():
             input_tree = config['input']
@@ -104,11 +119,13 @@ class BabyConfigParser:
 
             print('{}=== Handling output tree {} ==={}'.format(
                 TC.BOLD+TC.BLUE, output_tree, TC.END))
+            directive['input_trees'].append(input_tree)
 
             # Merge raw tree-specific directive with the global one.
             merge = config['inherit'] if 'inherit' in config else True
             config = update_config(self.parsed_config, config, merge=merge)
             namespace = defaultdict(dict)
+            namespace['literals'] = parsed_literals
             namespace['raw'] = {n: BabyVariable(n, t, input=True, output=False)
                                 for n, t in dumped_tree.items()}
 
@@ -127,14 +144,14 @@ class BabyConfigParser:
 
             # Resolve variables needed for selection
             selection, unresolved_selection = resolver.resolve_scope(
-                'selection', ['calculation', 'rename', 'raw'])
+                'selection', ['literals', 'calculation', 'rename', 'raw'])
 
             # Resolve all other variables
             keep, unresolved_keep = resolver.resolve_scope('keep', ['raw'])
             rename, unresolved_rename = resolver.resolve_scope(
                 'rename', ['raw'])
             calculation, unresolved_calculation = resolver.resolve_scope(
-                'calculation', ['calculation', 'rename', 'raw'])
+                'calculation', ['literals', 'calculation', 'rename', 'raw'])
             resolved_vars = selection + keep + rename + calculation
             most_unresolved_vars = unresolved_keep + unresolved_rename + \
                 unresolved_calculation
@@ -250,11 +267,16 @@ class BabyConfigParser:
         """
         Parse ``selection`` section.
         """
+        selections = deepcopy(config['global_selection']) \
+            if 'global_selection' in config else []
+
         if 'selection' in config:
-            for idx, expr in enumerate(config['selection']):
-                namespace['selection']['sel'+str(idx)] = BabyVariable(
-                    'sel'+str(idx), rvalues=[expr],
-                    input=False, output=False, fake=True)
+            selections += config['selection']
+
+        for idx, expr in enumerate(selections):
+            namespace['selection']['sel'+str(idx)] = BabyVariable(
+                'sel'+str(idx), rvalues=[expr],
+                input=False, output=False, fake=True)
 
     @staticmethod
     def match(patterns, string, return_value=True):
@@ -278,23 +300,31 @@ class BabyMaker(BaseMaker):
     """
     ``babymaker`` class to glue parser and code generator together.
     """
-    def __init__(self, config_filename, ntuple_filename, template_filename,
-                 use_reformater=True):
+    def __init__(self, config_filename, ntuple_filename, friend_filenames,
+                 template_filename,
+                 use_reformatter=True):
         """
         Initialize with path to YAML file and ntuple file.
         """
         self.config_filename = config_filename
         self.ntuple_filename = ntuple_filename
+        self.friend_filenames = friend_filenames
         self.template_filename = template_filename
-        self.use_reformater = use_reformater
+        self.use_reformatter = use_reformatter
 
-    def gen(self, filename, debug=False):
+    def gen(self, filename, literals={}, blocked_trees=[], debug=False):
         """
         Generate C++ file based on inputs.
         """
         parsed_config = self.read(self.config_filename)
-        dumped_ntuple = self.dump(self.ntuple_filename)
-        directive = self.directive_gen(parsed_config, dumped_ntuple, debug)
+        dumped_ntuple, tree_relations = self.dump_ntuples(blocked_trees)
+        directive = self.directive_gen(
+            parsed_config, dumped_ntuple, literals, debug)
+
+        # Adding ntuple info to the directive
+        directive['ntuple'] = self.ntuple_filename
+        directive['friends'] = self.friend_filenames
+        directive['tree_relations'] = tree_relations
 
         with open(self.template_filename) as tmpl:
             macros = template_transformer(tmpl, directive)
@@ -303,14 +333,37 @@ class BabyMaker(BaseMaker):
 
         with open(filename, 'w') as f:
             f.write(''.join(output_cpp))
-        if self.use_reformater:
+        if self.use_reformatter:
             self.reformat(filename)
 
+    def dump_ntuples(self, blocked_trees=[]):
+        """
+        Dump main ntuple and all friend ntuples.
+        """
+        trees = self.dump(self.ntuple_filename)
+        # Remove blocked input trees
+        trees = {k: v for k, v in trees.items() if k not in blocked_trees}
+        tree_relations = {k: [] for k in trees}
+
+        for friend in self.friend_filenames:
+            friend_trees = self.dump(friend)
+
+            for t in trees:
+                in_friend = t in friend_trees
+                tree_relations[t].append(in_friend)
+
+                if in_friend:
+                    # Mark branches in friend trees as available
+                    trees[t].update(friend_trees[t])
+
+        return trees, tree_relations
+
     @staticmethod
-    def directive_gen(parsed_config, dumped_ntuple, debug=False):
+    def directive_gen(parsed_config, dumped_ntuple,
+                      literals={}, debug=False):
         """
         Generate data structure (``directive``) needed for the C++ macro
         template.
         """
-        parser = BabyConfigParser(parsed_config, dumped_ntuple, debug)
+        parser = BabyConfigParser(parsed_config, dumped_ntuple, literals, debug)
         return parser.parse()
