@@ -2,7 +2,7 @@
 #
 # Author: Yipeng Sun <syp at umd dot edu>
 # License: BSD 2-clause
-# Last Change: Sat Mar 13, 2021 at 04:48 PM +0100
+# Last Change: Mon Jun 21, 2021 at 04:24 PM +0200
 
 import yaml
 import pytest
@@ -20,10 +20,11 @@ from pyBabyMaker.io.TupleDump import PyTupleDump
 
 PWD = dirname(realpath(__file__))
 PARDIR = J(PWD, pardir)
-SAMPLE_YAML = J(PARDIR, 'samples', 'sample-babymaker.yml')
-SAMPLE_ROOT = J(PARDIR, 'samples', 'sample.root')
-SAMPLE_TMPL = J(PARDIR, 'pyBabyMaker', 'cpp_templates', 'babymaker.cpp')
-SAMPLE_CPP  = J(PARDIR, 'samples', 'sample_cpp', 'sample-babymaker.cpp')
+SAMPLE_YAML   = J(PARDIR, 'samples', 'sample-babymaker.yml')
+SAMPLE_ROOT   = '../samples/sample.root'
+SAMPLE_FRIEND = '../samples/sample_friend.root'
+SAMPLE_TMPL   = J(PARDIR, 'pyBabyMaker', 'cpp_templates', 'babymaker.cpp')
+SAMPLE_CPP    = J(PARDIR, 'samples', 'sample-babymaker.cpp')
 
 
 ######################
@@ -56,12 +57,14 @@ def test_BabyVariable_set_fname():
 
 def test_BabyMaker_cpp_gen(tmp_path):
     gen_cpp = tmp_path / "gen_cpp.cpp"
-    babymaker = BabyMaker(SAMPLE_YAML, SAMPLE_ROOT, SAMPLE_TMPL, False)
-    babymaker.gen(gen_cpp, debug=True)
+    babymaker = BabyMaker(SAMPLE_YAML, SAMPLE_ROOT, [SAMPLE_FRIEND],
+                          SAMPLE_TMPL, use_reformatter=False)
+    babymaker.gen(gen_cpp, literals={'pi': '3.14'}, debug=True)
     gen_cpp_content = [line.strip()
                        for line in gen_cpp.read_text().split('\n')[1:]]
 
     with open(SAMPLE_CPP, 'r') as f:
+        print(f)
         assert gen_cpp_content == [line.strip() for line in f.readlines()]
 
 
@@ -79,7 +82,7 @@ def load_files():
 
 @pytest.fixture
 def realistic_BabyConfigParser(load_files):
-    return BabyConfigParser(*load_files, debug=True)
+    return BabyConfigParser(*load_files, literals={'pi': '3.14'}, debug=True)
 
 
 def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
@@ -87,6 +90,7 @@ def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
 
     assert directive['trees']['ATuple']['input_tree'] == 'TupleB0/DecayTree'
     assert directive['trees']['ATuple']['input_br'] == [
+        'raw_Y_ISOLATION_BDT',
         'raw_Y_PT',
         'raw_Y_PE',
         'raw_runNumber',
@@ -98,6 +102,7 @@ def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
         'raw_D0_P',
     ]
     assert directive['trees']['ATuple']['input'] == [
+        BabyVariable('Y_ISOLATION_BDT', 'double', input=True, output=False),
         BabyVariable('Y_PT', 'double', input=True, output=False),
         BabyVariable('Y_PE', 'double', input=True, output=False),
         BabyVariable('runNumber', 'UInt_t', input=True, output=False),
@@ -117,8 +122,8 @@ def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
         BabyVariable('y_px', 'double', ['Y_PX']),
         BabyVariable('y_py', 'double', ['Y_PY']),
         BabyVariable('y_pz', 'double', ['Y_PZ']),
-        BabyVariable('RandStuff', 'double', ['TempStuff']),
-        BabyVariable('some_other_var', 'double', ['some_var']),
+        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
+        BabyVariable('some_other_var', 'double', ['some_var*pi']),
         BabyVariable('alt_def', 'double', ['b0_pe', 'Y_PE']),
     ]
     assert directive['trees']['ATuple']['tmp'] == [
@@ -136,13 +141,13 @@ def test_BabyConfigParser_parse_ATuple(realistic_BabyConfigParser):
         BabyVariable('y_py', 'double', ['Y_PY']),
         BabyVariable('y_pz', 'double', ['Y_PZ']),
         BabyVariable('TempStuff', 'double', ['D0_P+Y_PT'], output=False),
-        BabyVariable('RandStuff', 'double', ['TempStuff']),
+        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
         BabyVariable('some_var', 'double', ['y_pt + y_pz'], output=False),
-        BabyVariable('some_other_var', 'double', ['some_var']),
+        BabyVariable('some_other_var', 'double', ['some_var*pi']),
         BabyVariable('alt_def', 'double', ['b0_pe', 'Y_PE']),
     ]
     assert realistic_BabyConfigParser._resolvers[0]._resolved_names[0] == \
-        ('raw', 'Y_PT')
+        ('raw', 'Y_ISOLATION_BDT')
 
 
 def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
@@ -151,6 +156,7 @@ def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
     assert directive['trees']['AnotherTuple']['input_tree'] == \
         'TupleB0/DecayTree'
     assert directive['trees']['AnotherTuple']['input'] == [
+        BabyVariable('Y_ISOLATION_BDT', 'double', input=True, output=False),
         BabyVariable('Y_PT', 'double', input=True, output=False),
         BabyVariable('Y_PE', 'double', input=True, output=False),
         BabyVariable('Y_PX', 'double', input=True, output=False),
@@ -173,7 +179,7 @@ def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
         BabyVariable('runNumber', 'UInt_t', ['runNumber']),
         BabyVariable('eventNumber', 'ULong64_t', ['eventNumber']),
         BabyVariable('GpsTime', 'ULong64_t', ['GpsTime']),
-        BabyVariable('RandStuff', 'double', ['TempStuff']),
+        BabyVariable('RandStuff', 'double', ['TempStuff*pi']),
     ]
     assert directive['trees']['AnotherTuple']['tmp'] == [
         BabyVariable('TempStuff', 'double', ['D0_P+Y_PT'], output=False),
@@ -181,6 +187,7 @@ def test_BabyConfigParser_parse_AnotherTuple(realistic_BabyConfigParser):
     assert directive['system_headers'] == ['cmath', 'iostream']
     assert directive['trees']['AnotherTuple']['sel'] == [
         'true',
+        'raw_Y_ISOLATION_BDT > 0',
         'rename_b0_pt > 10000',
         'raw_Y_PE > (100 * pow(10, 3))'
     ]
@@ -196,6 +203,7 @@ def test_BabyConfigParser_parse_YetAnotherTuple(realistic_BabyConfigParser):
     assert 'raw_Y_ISOLATION_NNp3' in input_br
     assert directive['trees']['YetAnotherTuple']['sel'] == [
         'true',
+        'raw_Y_ISOLATION_BDT > 0',
         'raw_piminus_isMuon'
     ]
 
@@ -210,6 +218,7 @@ def directive():
         'system_headers': UniqueList(),
         'user_headers': UniqueList(),
         'tree': {},
+        'input_trees': [],
     }
 
 
